<?php

namespace Drupal\drupal_contributions;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Link;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Url;
use Drupal\drupal_contributions\Contribution\ContributionClientInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides an organization's Drupal contribution issue credits.
 */
class ContributionCreditApi {

  use StringTranslationTrait;

  /**
   * The Contribution Client service.
   *
   * @var \Drupal\drupal_contributions\Contribution\ContributionClientInterface
   */
  protected $contributionClient;

  /**
   * The cache backend service.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cacheBackend;

  /**
   * Organization Group ID on Drupal.org.
   *
   * @var int
   */
  protected $organizationId;

  /**
   * Fetches and caches Drupal contribution data.
   *
   * @param \Drupal\drupal_contributions\Contribution\ContributionClientInterface $contribution_client
   *   The contribution client object to use.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The string translation service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(
    ContributionClientInterface $contribution_client,
    CacheBackendInterface $cache_backend,
    TranslationInterface $string_translation,
    ConfigFactoryInterface $config_factory
  ) {
    $this->contributionClient = $contribution_client;
    $this->cacheBackend = $cache_backend;
    $this->stringTranslation = $string_translation;
    $config = $config_factory->get('drupal_contributions.settings');
    $this->organizationId = $config->get('organization_id');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('drupal_contributions.contribution_client'),
      $container->get('cache.contribution_cache'),
      $container->get('string_translation')
    );
  }

  /**
   * Fetch contribution credits via ContributionClient.
   *
   * @return array
   *   Return an array of issue credits for the contributing organization.
   */
  public function getCreditsFetch() {
    $cid = 'contribution_credits_cache';
    // Restore projects from cache if available.
    if ($cached_data = $this->getCreditsCache()) {
      $projects = $cached_data['data'];
      $tags = $cached_data['tags'];
    }
    // Fetch all projects using the ContributionClient, then cache the data.
    else {
      $all_projects = $this->contributionClient->fetchProjects(
        $this->organizationId
      );
      // Set the cache tags.
      $tags = isset($all_projects['#cache']['tags']) ? $all_projects['#cache']['tags'] : [];
      $tags = Cache::mergeTags($tags, [$cid]);
      // Initialize arrays.
      $projects_core = [];
      $projects_contrib = [];
      // Loop through the fetched projects and create a list render array.
      foreach ($all_projects as $project_name => $values) {
        $issues = [];
        $project_link = Link::fromTextAndUrl($project_name, Url::fromUri($values['url']));
        // Create a list of issues per project.
        foreach ($values['issues'] as $issue) {
          $issue_link = Link::fromTextAndUrl($issue['title'], Url::fromUri($issue['url']));
          // Append the issue link with date to the issues array.
          $issues[] = [
            '#markup' => $this->t('%issue - @date', [
              '%issue' => $issue_link->toString(),
              '@date' => $issue['date'],
            ]),
          ];
        }
        // Save issue credits for Drupal core + contrib.
        if ($project_name == 'drupal') {
          $projects_core[] = [
            '#theme' => 'item_list',
            '#title' => $project_link,
            '#items' => $issues,
          ];
        }
        else {
          // Save issue credits for Drupal contributed modules.
          $projects_contrib[] = [
            '#theme' => 'item_list',
            '#title' => $project_link,
            '#items' => $issues,
          ];
        }
      }
      // Save core + contrib issues to the projects array.
      $projects['core'] = [
        '#theme' => 'item_list',
        '#title' => 'Chromatic Core Contributions',
        '#items' => $projects_core,
      ];
      $projects['contrib'] = [
        '#theme' => 'item_list',
        '#title' => 'Chromatic Contributed Module Contributions',
        '#items' => $projects_contrib,
      ];
      // Store the projects render array into the cache.
      $this->cacheBackend->set($cid, $projects, CacheBackendInterface::CACHE_PERMANENT, $tags);
    }
    // Set the cache tags.
    $projects['#cache'] = [
      'tags' => $tags,
    ];
    return $projects;
  }

  /**
   * Fetch contribution credits from cache.
   *
   * @return array|bool
   *   If the cached data exists, returns an array containing issue credits.
   *   Otherwise, returns false.
   */
  public function getCreditsCache() {
    if ($cached_data = $this->cacheBackend->get('contribution_credits_cache')) {
      return [
        'data' => $cached_data->data,
        'tags' => $cached_data->tags,
      ];
    }
    else {
      return FALSE;
    }
  }

}
